@@ -1,12 +1,14 @@
 from abc import ABC, abstractmethod
+from collections import defaultdict
+
 from sigma.exceptions import SigmaError, SigmaValueError
 from sigma.conversion.deferred import DeferredQueryExpression
-from typing import Union, ClassVar, Optional, Tuple, List, Dict, Any
+from typing import Pattern, Union, ClassVar, Optional, Tuple, List, Dict, Any
 from sigma.processing.pipeline import ProcessingPipeline
 from sigma.collection import SigmaCollection
 from sigma.rule import SigmaRule
-from sigma.conditions import ConditionItem, ConditionOR, ConditionAND, ConditionNOT, ConditionFieldEqualsValueExpression, ConditionFieldValueInExpression, ConditionValueExpression, ConditionType
-from sigma.types import SigmaBool, SigmaString, SigmaNumber, SigmaRegularExpression, SigmaCompareExpression, SigmaNull, SigmaQueryExpression, SigmaCIDRExpression
+from sigma.conditions import ConditionItem, ConditionOR, ConditionAND, ConditionNOT, ConditionFieldEqualsValueExpression, ConditionValueExpression, ConditionType
+from sigma.types import SigmaBool, SigmaExpansion, SigmaString, SigmaNumber, SigmaRegularExpression, SigmaCompareExpression, SigmaNull, SigmaQueryExpression, SigmaCIDRExpression, SpecialChars
 from sigma.conversion.state import ConversionState
 
 class Backend(ABC):
@@ -57,13 +59,19 @@ class Backend(ABC):
     """
     processing_pipeline : ProcessingPipeline
     backend_processing_pipeline : ClassVar[ProcessingPipeline] = ProcessingPipeline()
+    output_format_processing_pipeline : ClassVar[Dict[str, ProcessingPipeline]] = defaultdict(ProcessingPipeline)
     config : Dict[str, Any]
     default_format : ClassVar[str] = "default"
     collect_errors : bool = False
     errors : List[Tuple[SigmaRule, SigmaError]] = list()
 
+    # in-expressions
+    convert_or_as_in : ClassVar[bool] = False                     # Convert OR as in-expression
+    convert_and_as_in : ClassVar[bool] = False                    # Convert AND as in-expression
+    in_expressions_allow_wildcards : ClassVar[bool] = False       # Values in list can contain wildcards. If set to False (default) only plain values are converted into in-expressions.
+
     def __init__(self, processing_pipeline : Optional[ProcessingPipeline] = None, collect_errors : bool = False, **kwargs):
-        self.processing_pipeline = self.backend_processing_pipeline + processing_pipeline
+        self.processing_pipeline = processing_pipeline
         self.collect_errors = collect_errors
         self.config = kwargs
 
@@ -86,13 +94,15 @@ class Backend(ABC):
         """
         state = ConversionState()
         try:
-            self.processing_pipeline.apply(rule)        # 1. Apply transformations
+            processing_pipeline = self.backend_processing_pipeline + self.processing_pipeline + self.output_format_processing_pipeline[output_format or self.default_format]
+            processing_pipeline.apply(rule)             # 1. Apply transformations
+            state.processing_state = processing_pipeline.state
             queries = [                                 # 2. Convert condition
                 self.convert_condition(cond.parsed, state)
                 for cond in rule.detection.parsed_condition
             ]
             return [                                    # 3. Postprocess generated query
-                self.finalize_query(rule, query, index, state, output_format)
+                self.finalize_query(rule, query, index, state, output_format or self.default_format)
                 for index, query in enumerate(queries)
             ]
         except SigmaError as e:
@@ -102,6 +112,64 @@ class Backend(ABC):
             else:
                 raise e
 
+    def decide_convert_condition_as_in_expression(self, cond : Union[ConditionOR, ConditionAND], state : ConversionState) -> bool:
+        """
+        Decide if an OR or AND expression should be converted as "field in (value list)" or as plain expression.
+
+        :param cond: Condition that is converted for which the decision has to be made.
+        :type cond: Union[ConditionOR, ConditionAND]
+        :param state: Current conversion state.
+        :type state: ConversionState
+        :return: True if in-expression should be generated, else False
+        :rtype: bool
+        """
+        # Check if conversion of condition type is enabled
+        if (not self.convert_or_as_in and isinstance(cond, ConditionOR)
+           or not self.convert_and_as_in and isinstance(cond, ConditionAND)):
+           return False
+
+        # Check if more than one argument is present
+        #if len(cond.args <= 1):
+        #    return False
+
+        # All arguments of the given condition must reference a field
+        if not all((
+            isinstance(arg, ConditionFieldEqualsValueExpression)
+            for arg in cond.args
+        )):
+            return False
+
+        # Build a set of all fields appearing in condition arguments
+        fields = {
+            arg.field
+            for arg in cond.args
+        }
+        # All arguments must reference the same field
+        if len(fields) != 1:
+            return False
+
+        # All argument values must be strings or numbers
+        if not all([
+            isinstance(arg.value, ( SigmaString, SigmaNumber ))
+            for arg in cond.args
+        ]):
+           return False
+
+        # Check for plain strings if wildcards are not allowed for string expressions.
+        if not self.in_expressions_allow_wildcards and any([
+            arg.value.contains_special()
+            for arg in cond.args
+            if isinstance(arg.value, SigmaString)
+        ]):
+           return False
+
+        # All checks passed, expression can be converted to in-expression
+        return True
+
+    @abstractmethod
+    def convert_condition_as_in_expression(self, cond : Union[ConditionOR, ConditionAND], state : ConversionState) -> Any:
+        """Conversion of OR or AND conditions into "field in (value list)" expressions."""
+
     @abstractmethod
     def convert_condition_or(self, cond : ConditionOR, state : ConversionState) -> Any:
         """Conversion of OR conditions."""
@@ -146,6 +214,17 @@ class Backend(ABC):
     def convert_condition_field_eq_query_expr(self, cond : ConditionFieldEqualsValueExpression, state : ConversionState) -> Any:
         """Conversion of query expressions bound to a field."""
 
+    def convert_condition_field_eq_expansion(self, cond : ConditionFieldEqualsValueExpression, state : ConversionState) -> Any:
+        """
+        Convert each value of the expansion with the field from the containing condition and OR-link
+        all converted subconditions.
+        """
+        or_cond = ConditionOR([
+            ConditionFieldEqualsValueExpression(cond.field, value)
+            for value in cond.value.values
+        ], cond.source)
+        return self.convert_condition_or(or_cond, state)
+
     def convert_condition_field_eq_val(self, cond : ConditionFieldEqualsValueExpression, state : ConversionState) -> Any:
         """Conversion dispatcher of field = value conditions. Dispatches to value-specific methods."""
         if isinstance(cond.value, SigmaString):
@@ -164,18 +243,11 @@ class Backend(ABC):
             return self.convert_condition_field_eq_val_null(cond, state)
         elif isinstance(cond.value, SigmaQueryExpression):
             return self.convert_condition_field_eq_query_expr(cond, state)
+        elif isinstance(cond.value, SigmaExpansion):
+            return self.convert_condition_field_eq_expansion(cond, state)
         else:       # pragma: no cover
             raise TypeError("Unexpected value type class in condition parse tree: " + cond.value.__class__.__name__)
 
-    @abstractmethod
-    def convert_condition_field_in_vals(self, cond : ConditionFieldValueInExpression, state : ConversionState) -> Any:
-        """
-        Conversion of field in value conditions.
-
-        The value list will only contain plain strings (without wildcards) or numbers. These
-        both types must be handled accordingly.
-        """
-
     @abstractmethod
     def convert_condition_val_str(self, cond : ConditionValueExpression, state : ConversionState) -> Any:
         """Conversion of string-only conditions."""
@@ -223,15 +295,19 @@ class Backend(ABC):
         finalize_query method and must be implemented individually.
         """
         if isinstance(cond, ConditionOR):
-            return self.convert_condition_or(cond, state)
+            if self.decide_convert_condition_as_in_expression(cond, state):
+                return self.convert_condition_as_in_expression(cond, state)
+            else:
+                return self.convert_condition_or(cond, state)
         elif isinstance(cond, ConditionAND):
-            return self.convert_condition_and(cond, state)
+            if self.decide_convert_condition_as_in_expression(cond, state):
+                return self.convert_condition_as_in_expression(cond, state)
+            else:
+                return self.convert_condition_and(cond, state)
         elif isinstance(cond, ConditionNOT):
             return self.convert_condition_not(cond, state)
         elif isinstance(cond, ConditionFieldEqualsValueExpression):
             return self.convert_condition_field_eq_val(cond, state)
-        elif isinstance(cond, ConditionFieldValueInExpression):
-            return self.convert_condition_field_in_vals(cond, state)
         elif isinstance(cond, ConditionValueExpression):
             return self.convert_condition_val(cond, state)
         else:       # pragma: no cover
@@ -286,6 +362,13 @@ class TextQueryBackend(Backend):
     eq_token  : ClassVar[Optional[str]] = None      # Token inserted between field and value (without separator)
 
     # String output
+    ## Fields
+    field_quote : ClassVar[Optional[str]] = None                # Character used to quote field characters if field_quote_pattern matches (or not, depending on field_quote_pattern_negation). No field name quoting is done if not set.
+    field_quote_pattern : ClassVar[Optional[Pattern]] = None    # Quote field names if this pattern (doesn't) matches, depending on field_quote_pattern_negation. Field name is always quoted if pattern is not set.
+    field_quote_pattern_negation : ClassVar[bool] = True        # Negate field_quote_pattern result. Field name is quoted if pattern doesn't matches if set to True (default).
+    # TODO: add field name escaping at least for quote character
+
+    ## Values
     str_quote       : ClassVar[Optional[str]] = None    # string quoting character (added as escaping character)
     escape_char     : ClassVar[Optional[str]] = None    # Escaping character for special characrers inside string
     wildcard_multi  : ClassVar[Optional[str]] = None    # Character used as multi-character wildcard
@@ -297,6 +380,12 @@ class TextQueryBackend(Backend):
         False: None,
     }
 
+    # String matching operators. if none is appropriate eq_token is used.
+    startswith_expression : ClassVar[Optional[str]] = None
+    endswith_expression   : ClassVar[Optional[str]] = None
+    contains_expression   : ClassVar[Optional[str]] = None
+    wildcard_match_expression : ClassVar[Optional[str]] = None      # Special expression if wildcards can't be matched with the eq_token operator
+
     # Regular expressions
     re_expression : ClassVar[Optional[str]] = None      # Regular expression query as format string with placeholders {field} and {regex}
     re_escape_char : ClassVar[Optional[str]] = None     # Character used for escaping in regular expressions
@@ -314,8 +403,10 @@ class TextQueryBackend(Backend):
     # Null/None expressions
     field_null_expression : ClassVar[Optional[str]] = None          # Expression for field has null value as format string with {field} placeholder for field name
 
-    # Field value in list
-    field_in_list_expression : ClassVar[Optional[str]] = None       # Expression for field in list of values as format string with placeholders {field} and {list}
+    # Field value in list, e.g. "field in (value list)" or "field containsall (value list)"
+    field_in_list_expression : ClassVar[Optional[str]] = None       # Expression for field in list of values as format string with placeholders {field}, {op} and {list}
+    or_in_operator : ClassVar[Optional[str]] = None      # Operator used to convert OR into in-expressions. Must be set if convert_or_as_in is set
+    and_in_operator : ClassVar[Optional[str]] = None     # Operator used to convert AND into in-expressions. Must be set if convert_and_as_in is set
     list_separator : ClassVar[Optional[str]] = None     # List element separator
 
     # Value not bound to a field
@@ -323,23 +414,30 @@ class TextQueryBackend(Backend):
     unbound_value_num_expression : ClassVar[Optional[str]] = None   # Expression for number value not bound to a field as format string with placeholder {value}
     unbound_value_re_expression : ClassVar[Optional[str]] = None   # Expression for regular expression not bound to a field as format string with placeholder {value}
 
-    # Query finalization: appending and concatenating deferred query party
+    # Query finalization: appending and concatenating deferred query part
     deferred_start : ClassVar[Optional[str]] = None                 # String used as separator between main query and deferred parts
     deferred_separator : ClassVar[Optional[str]] = None             # String used to join multiple deferred query parts
     deferred_only_query : ClassVar[Optional[str]] = None            # String used as query if final query only contains deferred expression
 
-    def compare_precedence(self, outer : ConditionItem, inner : ConditionType) -> bool:
+    def compare_precedence(self, outer : ConditionItem, inner : ConditionItem) -> bool:
         """
         Compare precedence of outer and inner condition items. Return True if precedence of
         enclosing condition item (outer) is lower than the contained (inner) condition item.
         In this case, no additional grouping is required.
         """
+        outer_class = outer.__class__
+        # Special case: Conditions containing a SigmaExpansion value convert into OR conditions and therefore the precedence has to be handled the same way.
+        if isinstance(inner, ( ConditionFieldEqualsValueExpression, ConditionValueExpression )) and isinstance(inner.value, SigmaExpansion):
+            inner_class = ConditionOR
+        else:
+            inner_class = inner.__class__
+
         try:
-            idx_inner = self.precedence.index(inner)
+            idx_inner = self.precedence.index(inner_class)
         except ValueError:      # ConditionItem not in precedence tuple
             idx_inner = -1      # Assume precedence of inner condition item is higher than the outer
 
-        return idx_inner <= self.precedence.index(outer)
+        return idx_inner <= self.precedence.index(outer_class)
 
     def convert_condition_group(self, cond : ConditionItem, state : ConversionState) -> Union[str, DeferredQueryExpression]:
         """Group condition item."""
@@ -356,7 +454,7 @@ class TextQueryBackend(Backend):
             return joiner.join((
                     converted
                     for converted in (
-                        self.convert_condition(arg, state) if self.compare_precedence(ConditionOR, arg.__class__)
+                        self.convert_condition(arg, state) if self.compare_precedence(cond, arg)
                         else self.convert_condition_group(arg, state)
                         for arg in cond.args
                     )
@@ -365,6 +463,19 @@ class TextQueryBackend(Backend):
         except TypeError:       # pragma: no cover
             raise NotImplementedError("Operator 'or' not supported by the backend")
 
+    def convert_condition_as_in_expression(self, cond : Union[ConditionOR, ConditionAND], state : ConversionState) -> Union[str, DeferredQueryExpression]:
+        """Conversion of field in value list conditions."""
+        return self.field_in_list_expression.format(
+            field=self.quote_field(cond.args[0].field),       # The assumption that the field is the same for all argument is valid because this is checked before
+            op=self.or_in_operator if isinstance(cond, ConditionOR) else self.and_in_operator,
+            list=self.list_separator.join([
+                self.str_quote + self.convert_value_str(arg.value, state) + self.str_quote
+                if isinstance(arg.value, SigmaString)   # string escaping and qouting
+                else str(arg.value)       # value is number
+                for arg in cond.args
+            ]),
+        )
+
     def convert_condition_and(self, cond : ConditionAND, state : ConversionState) -> Union[str, DeferredQueryExpression]:
         """Conversion of AND conditions."""
         try:
@@ -376,7 +487,7 @@ class TextQueryBackend(Backend):
             return joiner.join((
                     converted
                     for converted in (
-                        self.convert_condition(arg, state) if self.compare_precedence(ConditionAND, arg.__class__)
+                        self.convert_condition(arg, state) if self.compare_precedence(cond, arg)
                         else self.convert_condition_group(arg, state)
                         for arg in cond.args
                     )
@@ -400,6 +511,21 @@ class TextQueryBackend(Backend):
         except TypeError:       # pragma: no cover
             raise NotImplementedError("Operator 'not' not supported by the backend")
 
+    def quote_field(self, field_name : str) -> str:
+        """Quote field name with field_quote if field_quote_pattern (doesn't) matches, depending on
+        field_quote_pattern_negation."""
+        if self.field_quote is not None:
+            if self.field_quote_pattern is not None:
+                quote = bool(self.field_quote_pattern.match(field_name))
+                if self.field_quote_pattern_negation:
+                    quote = not quote
+            else:
+                quote = True
+
+            if quote:
+                return self.field_quote + field_name + self.field_quote
+        return field_name
+
     def convert_value_str(self, s : SigmaString, state : ConversionState) -> Union[str, DeferredQueryExpression]:
         """Convert a SigmaString into a plain string which can be used in query."""
         return s.convert(
@@ -413,21 +539,52 @@ class TextQueryBackend(Backend):
     def convert_condition_field_eq_val_str(self, cond : ConditionFieldEqualsValueExpression, state : ConversionState) -> Union[str, DeferredQueryExpression]:
         """Conversion of field = string value expressions"""
         try:
-            return cond.field + self.eq_token + self.str_quote + self.convert_value_str(cond.value, state) + self.str_quote
+            if (                                                                # Check conditions for usage of 'startswith' operator
+                self.startswith_expression is not None                            # 'startswith' operator is defined in backend
+                and cond.value.endswith(SpecialChars.WILDCARD_MULTI)            # String ends with wildcard
+                and not cond.value[:-1].contains_special()                      # Remainder of string doesn't contains special characters
+                ):
+                expr = self.startswith_expression                               # If all conditions are fulfilled, use 'startswith' operartor instead of equal token
+                value = cond.value[:-1]
+            elif (                                                              # Same as above but for 'endswith' operator: string starts with wildcard and doesn't contains further special characters
+                self.endswith_expression is not None
+                and cond.value.startswith(SpecialChars.WILDCARD_MULTI)
+                and not cond.value[1:].contains_special()
+                ):
+                expr = self.endswith_expression
+                value = cond.value[1:]
+            elif (                                                              # contains: string starts and ends with wildcard
+                self.contains_expression is not None
+                and cond.value.startswith(SpecialChars.WILDCARD_MULTI)
+                and cond.value.endswith(SpecialChars.WILDCARD_MULTI)
+                and not cond.value[1:-1].contains_special()
+                ):
+                expr = self.contains_expression
+                value = cond.value[1:-1]
+            elif (                                                              # wildcard match expression: string contains wildcard
+                self.wildcard_match_expression is not None
+                and cond.value.contains_special()
+                ):
+                expr = self.wildcard_match_expression
+                value = cond.value
+            else:
+                expr =  "{field}" + self.eq_token + self.str_quote + "{value}" + self.str_quote # [New]
+                value = cond.value
+            return expr.format(field=self.quote_field(cond.field), value=self.convert_value_str(value, state))
         except TypeError:       # pragma: no cover
             raise NotImplementedError("Field equals string value expressions with strings are not supported by the backend.")
 
     def convert_condition_field_eq_val_num(self, cond : ConditionFieldEqualsValueExpression, state : ConversionState) -> Union[str, DeferredQueryExpression]:
         """Conversion of field = number value expressions"""
         try:
-            return cond.field + self.eq_token + str(cond.value)
+            return self.quote_field(cond.field) + self.eq_token + str(cond.value)
         except TypeError:       # pragma: no cover
             raise NotImplementedError("Field equals numeric value expressions are not supported by the backend.")
 
     def convert_condition_field_eq_val_bool(self, cond : ConditionFieldEqualsValueExpression, state : ConversionState) -> Union[str, DeferredQueryExpression]:
         """Conversion of field = bool value expressions"""
         try:
-            return cond.field + self.eq_token + self.bool_values[cond.value.boolean]
+            return self.quote_field(cond.field) + self.eq_token + self.bool_values[cond.value.boolean]
         except TypeError:       # pragma: no cover
             raise NotImplementedError("Field equals numeric value expressions are not supported by the backend.")
 
@@ -442,14 +599,14 @@ class TextQueryBackend(Backend):
     def convert_condition_field_eq_val_re(self, cond : ConditionFieldEqualsValueExpression, state : ConversionState) -> Union[str, DeferredQueryExpression]:
         """Conversion of field matches regular expression value expressions."""
         return self.re_expression.format(
-            field=cond.field,
+            field=self.quote_field(cond.field),
             regex=self.convert_value_re(cond.value, state),
         )
 
     def convert_condition_field_eq_val_re_contains(self, cond : ConditionFieldEqualsValueExpression, state : ConversionState) -> Union[str, DeferredQueryExpression]:
         """Conversion of value-only regular expressions."""
         return self.re_expression.format(
-            field=cond.field,
+            field=self.quote_field(cond.field),
             regex=self.convert_value_re(cond.value, state),
         )
 
@@ -460,52 +617,41 @@ class TextQueryBackend(Backend):
             list_ip = convert_str.split(self.or_token)
             if self.cidr_wildcard == None:
                     return self.cidr_in_list_expression.format(
-                        field=cond.field,
+                        field=self.quote_field(cond.field),
                         list=self.list_separator.join([str(v) for v in list_ip])
                     )
             else:
                 return self.cidr_in_list_expression.format(
-                    field=cond.field,
+                    field=self.quote_field(cond.field),
                     list=self.list_separator.join([ self.str_quote + str(v) + self.str_quote for v in list_ip])
                 )
         else:
             if self.cidr_wildcard == None:
                 return self.cidr_expression.format(
-                    field=cond.field,
+                    field=self.quote_field(cond.field),
                     value=convert_str,
                 )
             else:
                 return self.cidr_expression.format(
-                    field=cond.field,
+                    field=self.quote_field(cond.field),
                     value=self.str_quote + convert_str + self.str_quote,
                 )
 
     def convert_condition_field_compare_op_val(self, cond : ConditionFieldEqualsValueExpression, state : ConversionState) -> Union[str, DeferredQueryExpression]:
         """Conversion of numeric comparison operations into queries."""
         return self.compare_op_expression.format(
-            field=cond.field,
+            field=self.quote_field(cond.field),
             operator=self.compare_operators[cond.value.op],
             value=cond.value.number,
         )
 
     def convert_condition_field_eq_val_null(self, cond : ConditionFieldEqualsValueExpression, state : ConversionState) -> Union[str, DeferredQueryExpression]:
         """Conversion of field is null expression value expressions"""
-        return self.field_null_expression.format(field=cond.field)
+        return self.field_null_expression.format(field=self.quote_field(cond.field))
 
     def convert_condition_field_eq_query_expr(self, cond : ConditionFieldEqualsValueExpression, state : ConversionState) -> Union[str, DeferredQueryExpression]:
         """Conversion of field is null expression value expressions"""
-        return cond.value.finalize(field=cond.field)
-
-    def convert_condition_field_in_vals(self, cond : ConditionFieldValueInExpression, state : ConversionState) -> Union[str, DeferredQueryExpression]:
-        """Conversion of field in value list conditions."""
-        return self.field_in_list_expression.format(
-            field=cond.field,
-            list=self.list_separator.join([
-                self.str_quote + self.convert_value_str(v, state) + self.str_quote if isinstance(v, SigmaString)   # string escaping and qouting
-                else str(v)       # value is number
-                for v in cond.value
-            ]),
-        )
+        return cond.value.finalize(field=self.quote_field(cond.field))
 
     def convert_condition_val_str(self, cond : ConditionValueExpression, state : ConversionState) -> Union[str, DeferredQueryExpression]:
         """Conversion of value-only strings."""
@@ -520,7 +666,7 @@ class TextQueryBackend(Backend):
         return self.unbound_value_re_expression.format(value=self.convert_value_re(cond.value, state))
 
     def convert_condition_query_expr(self, cond : ConditionValueExpression, state : ConversionState) -> Union[str, DeferredQueryExpression]:
-        """Conversion of value-only regular expressions."""
+        """Conversion of value-only plain query expressions."""
         return cond.value.finalize()
 
     def finalize_query(self, rule : SigmaRule, query : Union[str, DeferredQueryExpression], index : int, state : ConversionState, output_format : str) -> Union[str, DeferredQueryExpression]:
@@ -540,4 +686,4 @@ class TextQueryBackend(Backend):
                 index, state, output_format
             )
         else:
-            return super().finalize_query(rule, query, index, state, output_format)
\ No newline at end of file
+            return super().finalize_query(rule, query, index, state, output_format)
