@@ -1,6 +1,6 @@
 from dataclasses import dataclass, field
 from sigma.conversion.deferred import DeferredQueryExpression
-from typing import List
+from typing import Any, List, Mapping
 
 @dataclass
 class ConversionState:
@@ -18,6 +18,7 @@ class ConversionState:
     down or up the parse tree.
     """
     deferred : List[DeferredQueryExpression] = field(default_factory=list)
+    processing_state : Mapping[str, Any] = field(default_factory=dict)
 
     def add_deferred_expression(self, deferred : DeferredQueryExpression) -> None:
         self.deferred.append(deferred)
