@@ -1,6 +1,6 @@
 from abc import ABC, abstractmethod
 from sigma.conditions import SigmaCondition
-from typing import Iterable, List, Dict, Optional, Union, Pattern, Iterator
+from typing import Any, Iterable, List, Dict, Optional, Union, Pattern, Iterator
 from dataclasses import dataclass, field
 import dataclasses
 import random
@@ -28,11 +28,12 @@ class Transformation(ABC):
     def apply(self, pipeline : "sigma.processing.pipeline.ProcessingPipeline", rule : SigmaRule) -> None:
         """Apply transformation on Sigma rule."""
         self.pipeline = pipeline        # make pipeline accessible from all further options in class property
+        self.processing_item_applied(rule)
 
     def set_processing_item(self, processing_item : "sigma.processing.pipeline.ProcessingItem"):
         self.processing_item = processing_item
 
-    def processing_item_applied(self, d : Union[SigmaDetection, SigmaDetectionItem, SigmaCondition]):
+    def processing_item_applied(self, d : Union[SigmaRule, SigmaDetection, SigmaDetectionItem, SigmaCondition]):
         """Mark detection item or detection as applied."""
         d.add_applied_processing_item(self.processing_item)
 
@@ -195,7 +196,8 @@ class AddFieldnameSuffixTransformation(DetectionItemTransformation):
     suffix : str
 
     def apply_detection_item(self, detection_item : SigmaDetectionItem):
-        detection_item.field += self.suffix
+        if type(detection_item.field) is str:
+            detection_item.field += self.suffix
         self.processing_item_applied(detection_item)
 
 @dataclass
@@ -313,7 +315,7 @@ class QueryExpressionPlaceholderTransformation(PlaceholderIncludeExcludeMixin, V
             if len(val.s) == 1:     # Sigma string must only contain placeholder, nothing else.
                 p = val.s[0]
                 if self.is_handled_placeholder(p):
-                    return SigmaQueryExpression(self.expression.format(field=field, id=self.mapping.get(p.name) or p.name))
+                    return SigmaQueryExpression(self.expression, self.mapping.get(p.name) or p.name)
             else:       # SigmaString contains placeholder as well as other parts
                 raise SigmaValueError(f"Placeholder query expression transformation only allows placeholder-only strings.")
         return None
@@ -370,6 +372,16 @@ class ReplaceStringTransformation(ValueTransformation):
         if isinstance(val, SigmaString):
             return SigmaString(self.re.sub(self.replacement, str(val)))
 
+@dataclass
+class SetStateTransformation(Transformation):
+    """Set pipeline state key to value."""
+    key : str
+    val : Any
+
+    def apply(self, pipeline: "sigma.processing.pipeline.Proces", rule: SigmaRule) -> None:
+        super().apply(pipeline, rule)
+        pipeline.state[self.key] = self.val
+
 @dataclass
 class RuleFailureTransformation(Transformation):
     """
@@ -405,6 +417,7 @@ transformations : Dict[str, Transformation] = {
     "add_condition": AddConditionTransformation,
     "change_logsource": ChangeLogsourceTransformation,
     "replace_string": ReplaceStringTransformation,
+    "set_state": SetStateTransformation,
     "rule_failure": RuleFailureTransformation,
     "detection_item_failure": DetectionItemFailureTransformation,
 }
